# $Id: LDAP.pm 29086 2009-10-23 12:03:54Z wsl $
# $URL: https://infix.uvt.nl/its-id/trunk/sources/aselect-perl/lib/Aselect/LDAP.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package Aselect::LDAP;

use Aselect::Login -Base;

use Net::LDAP;
use Net::LDAP::Filter;

field 'error';

sub authenticate {
	my ($uid, $passwd) = @_;

	return undef unless defined $uid;
	return undef unless defined $passwd;

	my $cfg = $self->cfg;

	# create a, possible cached, LDAP client object
	my $server = $cfg->ldap_server;
	$server = 'ldap://localhost'
		unless defined $server;

	my $ldap;
	our %ldaps;
	if(exists $ldaps{$server}) {
		$ldap = $ldaps{$server};
		eval { $ldap->bind };
		if($@) {
			warn "LDAP server $server: $@"
				unless $@ =~ /^Unexpected EOF /;
			undef $ldap;
			delete $ldaps{$server};
		}
	}
	unless(defined $ldap) {
		$ldap = new Net::LDAP($server, onerror => 'die', timeout => 10)
			or die "Connecting to $server: $!";
		$ldap->bind;
		$ldaps{$server} = $ldap;
	}

	my $search;

	# attribute to search on
	my $attribute = $cfg->ldap_attribute;
	$attribute = 'uid'
		unless defined $attribute;
	my $filterstruct = {equalityMatch => {attributeDesc => $attribute, assertionValue => $uid}};

	# create a, possible cached, filter
	my $filter = $cfg->ldap_filter;
	if(defined $filter) {
		our %filters;
		my $given;
		if(exists $filters{$filter}) {
			$given = $filters{$filter};
		} else {
			eval { $given = new Net::LDAP::Filter($filter); };
			die "Error parsing $filter: $@" if $@;
			$filters{$filter} = $given;
		}
		$search = bless({and => [$given, $filterstruct]}, 'Net::LDAP::Filter')->as_string;
	} else {
		$search = bless($filterstruct, 'Net::LDAP::Filter')->as_string;
	}

	my $base = $cfg->ldap_base;
	die "AselectLDAPBase not configured\n"
		unless defined $base;

	my $attributes = $cfg->attributes;

	my $res = $ldap->search(
			base => $base,
			filter => $search,
			attrs => [values %$attributes]
		);

	my %attrs;
	my $dn;

	foreach my $entry ($res->entries) {
		my $attempt = $entry->dn;
		eval {
			$ldap->bind($attempt, password => $passwd);
			$ldap->unbind;
			$dn = $attempt;
			while(my ($key, $val) = each(%$attributes)) {
				my $value = $entry->get_value($val);
				$attrs{$key} = $value if defined $value;
			}
		};
		last if defined $dn;
		warn "$attempt: $@";
	}

	return undef unless defined $dn;
	return \%attrs;
}

sub build {
	my $doc = $self->doc;

	my $info = $self->info;
	my $rid = $info->{rid};

	$self->addTemplates('login');

	my $ridinput = $doc->getElementById('rid');
	$ridinput->setAttribute('value', $rid);

	my $message = $self->error;
	if(defined $message) {
		my $err = $doc->getElementById('error');
		$err->appendText($message);
	}
}

sub login {
	my $info = $self->info;
	my $req = $self->req;
	my $cfg = $self->cfg;

	my $uid = $req->param('user_id');
	my $passwd = $req->param('password');

	return undef unless defined $uid && defined $passwd;

	$uid = lc $uid;

	my $attrs = $self->authenticate($uid, $passwd)
		or return undef;
	my $org = $cfg->organization;
	return {uid => $uid, organization => $org, attributes => $attrs}
		if %$attrs;
	return {uid => $uid, organization => $org};
}
